export link!
"""
Links the values of two observables by adding a listener. Optional arguments f and g specify transform of the value between the observables. If optional argument master=true all listeners of o2 are called if o1 is updated rather than just the value.

Example:
link(o1, o2, (x) -> 2*x, (x) -> x/2)
will ensure observe(o2)[] is always 2*observe(o1)[] and vice versa.

"""
function link!(o1::Observable, o2::Observable, f = (x)->(x), g = (x)->(x), master=false)
    if master
        push!(o1.listeners, (x) -> o2[]=f(o1[]))
    else
        push!(o1.listeners, (x) -> o2.val=f(o1[]))
    end
    push!(o2.listeners, (x) -> o1.val = g(o2[]))
end
    
